
! Used TDMA to solve implicitly
Module Grids
Type grid
  integer :: ni
  real :: dx
  real,dimension(:),allocatable :: xt0
  real,dimension(:),allocatable :: u
End type grid

Contains
Subroutine set_grid (grid_1d,ni)
type(grid) :: grid_1d
integer, intent(in) :: ni
integer :: i
real :: pi
pi = acos(-1.0)
grid_1d%ni = ni
grid_1d%dx = 1.0/(ni+1)
allocate (grid_1d%xt0(ni))
allocate (grid_1d%u(0:ni+1))
do i = 1,ni
  grid_1d%xt0(i) = i*grid_1d%dx
enddo
grid_1d%u(:) = 0.0
! set initial conditions as well
do i = 1,ni
  grid_1d%u(i) = sin(pi*(grid_1d%xt0(i)))
 enddo

End Subroutine set_grid
End Module Grids
!
Module netcdf_setup
use Grids


Contains
subroutine nc_setup (grid_1d,ncid,nc_filename)
integer,intent(inout) ::ncid
type (grid) :: grid_1d
integer :: ier
character(len=20),intent(inout)::nc_filename
character(len=20) ::fname
integer :: xdimid,timedimid
integer :: xid
integer :: timeid,tempid
include 'netcdf.inc'
!
fname = trim(nc_filename)

! For netcdf output
! define dimensions
ier=nf_create(fname,nf_clobber,ncid)
ier=NF_DEF_DIM(ncid,'x',grid_1d%ni,xdimid)
ier=NF_DEF_DIM(ncid,'time',nf_unlimited,timedimid)
ier=NF_DEF_VAR(ncid,'x',NF_REAL,1,(/xdimid/),xid)
ier=NF_DEF_VAR(ncid,'time',NF_REAL,1,(/timedimid/),timeid)
! define dependent variables
ier=NF_DEF_VAR(ncid,'Temp',NF_REAL,2,(/xdimid,timedimid/),tempid)
! attributes
ier=NF_PUT_ATT_TEXT(ncid,xid,'long_name',1,'x')
ier=NF_PUT_ATT_TEXT(ncid,xid,'units',1,'m')
ier=NF_PUT_ATT_TEXT(ncid,timeid,'long_name',10,'Julian Day')
ier=NF_PUT_ATT_TEXT(ncid,timeid,'units',31,'days since 1994-01-01 00:00:00')
ier=NF_PUT_ATT_TEXT(ncid,tempid,'units',6,'Kelvin')
!
ier=NF_ENDDEF(ncid)
!
ier=NF_PUT_VARA_REAL(ncid,xid,(/1/),(/grid_1d%ni/),grid_1d%xt0)
ier=nf_close(ncid)
!
End subroutine nc_setup
End Module netcdf_setup

Module update_temp
use Grids
Contains
subroutine update_u(grid_1d,dt)
type(grid) :: grid_1d
real,intent(in) ::dt
real,allocatable,dimension(:) :: a,b,c,d,sol,u
real :: alpha
integer :: ni
!
allocate(u(0:grid_1d%ni+1))

u(:) = grid_1d%u(:)
dx = grid_1d%dx
ni = grid_1d%ni
alpha = dt/dx**2
allocate (a(ni),b(ni),c(ni),d(ni),sol(ni))
do i = 1,ni
   a(i) = -alpha
   c(i) = -alpha
   b(i) = 1 + 2*alpha
   d(i) = u(i)
enddo
call tdma(a,b,c,ni,d,sol)
grid_1d%u(1:ni) = sol(1:ni)
deallocate(a,b,c,d,sol,u)
End Subroutine update_u
End Module update_temp

subroutine tdma(a,b,c,n,d,x)
! Subroutine to solve tridiagonal matrix a(i)x(i-1) + b(i)*x(i) + c(i)*x(i+1) = d(i)
! i = 2, n for a(i); i = 1,n for b(i) and d(i) ; i = 1, n-1 for c(i)
integer, intent(in) :: n
real, dimension(n)  :: a(n),b(n),c(n),d(n)
real,dimension(n),intent(out) :: x(n)
real :: denom
! Forward sweep
c(1) = c(1)/b(1)
d(1) = d(1)/b(1)
do i = 2,n-1
  denom = b(i) - c(i-1)*a(i)
  c(i) = c(i)/denom
  d(i) = (d(i) - d(i-1)*a(i))/denom
enddo
!
d(n) = (d(n) -d(n-1)*a(n))/(b(n) - c(n-1)*a(n))
x(n) =  d(n)
! backward sweep
do i = n-1,1,-1
  x(i) = d(i) - c(i)*x(i+1)
enddo
return 
end 
  


!
! Swathi..Test program to solve u<sub>t = delsq(u) on a unit square
! with zero bc's and sin(pi*x)*sin(pi*y) initial condition
PROGRAM  heat_eqn
  use Grids
  use netcdf_setup
  use update_temp
  implicit none
  integer :: ni, nt
  real, allocatable,dimension(:) :: usave
  real :: dt,dx,dt_cfl
  include 'netcdf.inc'
  integer :: ncid,ier
  character (len=20) ::nc_filename

  integer timeid,tempid,ltime
!
  type(grid) :: my_grid 
!
  integer :: i,n
!
  ni=16
  nt=500
  dt=8e-4
  nc_filename = 'heat_eqn_1D_tdma.nc'
!
  write(6,'(a,2i5,es12.4)' )'ni,nt,dt',ni,nt,dt
! Make some checks
  if ( ni <= 0 .OR. dt <= 0 .OR. nt <= 0) then
    write(6,*)'=>Error one of ni,nj,nt,dt is <= 0 stopping'
    stop
  endif
! Call gird and initialisation
  call set_grid(my_grid,ni)

! netcdf setup
  call nc_setup (my_grid,ncid,nc_filename)
!Loop over  time and write output 

  ltime=0
  allocate (usave(ni))
  ier=nf_open(trim(nc_filename),NF_WRITE,ncid)
  ier=nf_inq_varid(ncid,'Temp',tempid)
  ier=nf_inq_varid(ncid,'time',timeid)
  do n = 1,nt
    call update_u(my_grid,dt)
    if (mod(n,10) .eq.0) then
        ltime=ltime+1
        usave(1:my_grid%ni) = my_grid%u(1:my_grid%ni)
        ier=NF_PUT_VARA_REAL(ncid,tempid,(/1,ltime/),(/my_grid%ni,1/),usave)
        ier=NF_PUT_VARA_REAL(ncid,timeid,(/ltime/),(/1/),n*1.0)	

     endif
  enddo ! for n loop
  ier=nf_close(ncid)
END PROGRAM heat_eqn


